@@ -1,8 +1,7 @@
 #ifdef HAVE_CONFIG_H
-#include "dgconfig.h"
+#include "e2config.h"
 #endif
 
-#ifdef __SSLMITM
 #include <stdio.h>
 #include <stdlib.h>
 #include <sys/types.h>
@@ -12,7 +11,6 @@
 #include <unistd.h>
 #include <string.h>
 #include <errno.h>
-#include <syslog.h>
 
 #include <climits>
 #include <string>
@@ -29,27 +27,19 @@
 
 #include "CertificateAuthority.hpp"
 #include "OptionContainer.hpp"
-
+#include "Logger.hpp"
 
 extern OptionContainer o;
-extern thread_local std::string thread_id;
 
-void log_ssl_errors( const char *mess, const char *site) {
-    if( o.log_ssl_errors ) {
-        std::string tmess = thread_id;
-        tmess += mess;
-        syslog(LOG_ERR, tmess.c_str(), site);
-#ifdef DGDEBUG
-        std::cout  << thread_id << "SSL Error: " << mess << " at: " << site << std::endl;
-#endif
+void log_ssl_errors(const char *mess, const char *site) {
+    //E2LOGGER_debugnet(mess, site);
+    if( o.logger.log_ssl_errors ) {
+        E2LOGGER_error("SSL Error: ", mess, " at: ", site);
         unsigned long e;
         char buff[512];
         while ((e = ERR_get_error())) {
-           ERR_error_string(e, &buff[0]);
-           syslog(LOG_ERR, "%s%s", thread_id.c_str(), buff );
-#ifdef DGDEBUG
-        std::cout  << thread_id << "SSL Error: " << buff << " at: " << site << std::endl;
-#endif
+            ERR_error_string(e, &buff[0]);
+            E2LOGGER_error("SSL Error: ", buff, " at: ", site);
         }
     }
 }
@@ -62,33 +52,43 @@ CertificateAuthority::CertificateAuthority(const char *caCert,
     time_t caEnd)
 {
     FILE *fp;
+    char pem[7200];
 
     //load the ca cert
     fp = fopen(caCert, "r");
     if (fp == NULL) {
-        syslog(LOG_ERR, "%sCouldn't open ca certificate", thread_id.c_str());
+        E2LOGGER_error("Couldn't open ca certificate file ", caCert);
         exit(1);
     }
     _caCert = PEM_read_X509(fp, NULL, NULL, NULL);
     if (_caCert == NULL) {
-        //syslog(LOG_ERR, "Couldn't load ca certificate");
-        log_ssl_errors("Couldn't load ca certificate from %s",  caCert);
-        //ERR_print_errors_fp(stderr);
+        log_ssl_errors("Couldn't load ca certificate from ",  caCert);
         exit(1);
     }
+    fclose(fp);
 
+    // open again to get raw PEM for hash
+    fp = fopen(caCert, "r");
+    for (int i = 0;i < 7200;i++) {
+        pem[i] = '\0';
+    }
+    int rc = 0;
+    rc = fread(pem,72,99,fp);
+    if(rc < 1) {
+        E2LOGGER_error("Unable to re-read ca certificate file");
+        exit(1);
+    }
     fclose(fp);
 
     //load the ca priv key
     fp = fopen(caPrivKey, "r");
     if (fp == NULL) {
-        syslog(LOG_ERR, "%sCouldn't open ca private key", thread_id.c_str());
+        E2LOGGER_error("Couldn't open ca private key");
         exit(1);
     }
     _caPrivKey = PEM_read_PrivateKey(fp, NULL, NULL, NULL);
     if (_caPrivKey == NULL) {
-        syslog(LOG_ERR, "%sCouldn't load ca private key", thread_id.c_str());
-        //ERR_print_errors_fp(stderr);
+        E2LOGGER_error("Couldn't load ca private key");
         exit(1);
     }
 
@@ -97,14 +97,13 @@ CertificateAuthority::CertificateAuthority(const char *caCert,
     //load the priv key to use with generated certificates
     fp = fopen(certPrivKey, "r");
     if (fp == NULL) {
-        syslog(LOG_ERR, "%sCouldn't open certificate private key", thread_id.c_str());
+        E2LOGGER_error("Couldn't open certificate private key");
         exit(1);
     }
     _certPrivKey = PEM_read_PrivateKey(fp, NULL, NULL, NULL);
 
     if (_certPrivKey == NULL) {
-        syslog(LOG_ERR, "%sCouldn't load certificate private key", thread_id.c_str());
-        //ERR_print_errors_fp(stderr);
+        E2LOGGER_error("Couldn't load certificate private key");
         exit(1);
     }
     fclose(fp);
@@ -112,12 +111,23 @@ CertificateAuthority::CertificateAuthority(const char *caCert,
     //TODO should check this is a writable dir
     _certPath = certPath;
     _certPathLen = sizeof(certPath);
-    //	_certLinks = certLinks;
     _certLinks = certPath; // temp to check if this works
-    //_ca_start = 1417872951;  // 6th Dec 2014
-    //_ca_end = _ca_start + 315532800;  // 6th Dec 2024
     _ca_start = caStart;
     _ca_end = caEnd;
+
+    // Generate hash to be added to cert server name to produce serial unique rootCA_PEM/start_date/stop_date
+
+//   ASN1_TIME *not_before = X509_get_notBefore(_caCert);
+//   ASN1_TIME *not_after = X509_get_notAfter(_caCert);
+
+     // Make hash
+     String tem = pem;
+     String tem2((long)caStart), tem3((long)caEnd);
+     tem += tem2;
+     tem += tem3;
+     cert_start_stop_hash = tem.md5();
+     DEBUG_config("modifing string to be hashed (rootCA.pem + cartstart + certend is: ", tem);
+     DEBUG_config("modifing hash is: ", cert_start_stop_hash);
 }
 
 bool CertificateAuthority::getSerial(const char *commonname, struct ca_serial *caser)
@@ -126,22 +136,16 @@ bool CertificateAuthority::getSerial(const char *commonname, struct ca_serial *c
     char cnhash[EVP_MAX_MD_SIZE];
     unsigned int cnhashlen;
 
-    // added to generate different serial number than previous versions
-    //   needs to be added as an option
     std::string sname(commonname );
-    sname += "B";
+    sname += cert_start_stop_hash;
 
-#ifdef DGDEBUG
-    std::cout  << thread_id << "Generating serial no for " << commonname << std::endl;
-#endif
+    DEBUG_debug("Generating serial no for ", commonname );
 
     EVP_MD_CTX *mdctx;
 #if OPENSSL_VERSION_NUMBER < 0x10100000L
-    EVP_MD_CTX  mmdctx;
-    mdctx = &mmdctx;
-#else
-    mdctx = EVP_MD_CTX_new();
+#error "openssl version 1.1 or greater is required"
 #endif
+    mdctx = EVP_MD_CTX_new();
     const EVP_MD *md = EVP_md5();
     EVP_MD_CTX_init(mdctx);
 
@@ -158,11 +162,7 @@ bool CertificateAuthority::getSerial(const char *commonname, struct ca_serial *c
         failed = true;
     }
 
-#if OPENSSL_VERSION_NUMBER < 0x10100000L
-    EVP_MD_CTX_cleanup(mdctx);
-#else
     EVP_MD_CTX_free(mdctx);
-#endif
 
     if (failed) {
         return false;
@@ -176,13 +176,11 @@ bool CertificateAuthority::getSerial(const char *commonname, struct ca_serial *c
     }
 
     char *dbg = BN_bn2hex(bn);
-#ifdef DGDEBUG
     if (dbg != NULL) {
-        std::cout  << thread_id << "Serial no is " << dbg << std::endl;
+        DEBUG_debug("Serial no is ", dbg);
     } else {
-        std::cout  << thread_id << "bn2hex returned null instead of serial number" << std::endl;
+        DEBUG_debug("bn2hex returned null instead of serial number");
     }
-#endif
     caser->charhex = dbg;
     caser->asn = BN_to_ASN1_INTEGER(bn, NULL);
     BN_free(bn);
@@ -200,8 +198,8 @@ bool CertificateAuthority::writeCertificate(const char *commonname, X509 *newCer
     // make directory path
     int rc = mkpath(dirpath.c_str(), 0700); // only want e2g to have access to these dir
     if (rc != 0) {
-        syslog(LOG_ERR, "%serror creating certificate sub-directory: %s",  thread_id.c_str(),dirpath.c_str());
-        return false;
+        E2LOGGER_error("error creating certificate sub-directory: ", dirpath);
+        return false; 
     }
 
     //open file
@@ -212,13 +210,10 @@ bool CertificateAuthority::writeCertificate(const char *commonname, X509 *newCer
     fl.l_len = 0;
     fl.l_pid = getpid();
 
+    DEBUG_debug("certificate file is ",path);
     int fd = open(path.c_str(), O_RDWR | O_CREAT, S_IWUSR | S_IRUSR); //only e2g has access
-
-#ifdef DGDEBUG
-    std::cout  << thread_id << "certificate file is " << path << std::endl;
-#endif
     if (fd < 0) {
-        syslog(LOG_ERR, "%serror opening new certificate", thread_id.c_str());
+        E2LOGGER_error("error opening new certificate");
         exit(1);
     }
 
@@ -232,29 +227,23 @@ bool CertificateAuthority::writeCertificate(const char *commonname, X509 *newCer
 
     //check if someone else created the file before we did (avoid the race condition)
     if (pos < 0) {
-#ifdef DGDEBUG
-        std::cout  << thread_id << "error seeking to find certificate size " << std::endl;
-#endif
+        DEBUG_debug("error seeking to find certificate size ");
         fl.l_type = F_UNLCK;
         fcntl(fd, F_SETLK, &fl);
         close(fd);
         return false;
     } else if (pos > 0) {
-//didnt get first lock so cert should be there now
-#ifdef DGDEBUG
-        std::cout  << thread_id << "didnt get first lock pos was " << pos << std::endl;
-#endif
+        //didnt get first lock so cert should be there now
+        DEBUG_debug("didnt get first lock pos was ", pos);
         fl.l_type = F_UNLCK;
         fcntl(fd, F_SETLK, &fl);
         close(fd);
         return true;
     }
 
-//looks like we got the first lock so write the certificate
-//write the cert to a file
-#ifdef DGDEBUG
-    std::cout  << thread_id << "got first lock " << std::endl;
-#endif
+    //looks like we got the first lock so write the certificate
+    //write the cert to a file
+    DEBUG_debug("got first lock ");
     FILE *fp = fdopen(fd, "w");
     if (fp == NULL) {
         return false;
@@ -284,24 +273,18 @@ bool CertificateAuthority::writeCertificate(const char *commonname, X509 *newCer
 }
 
 //generate a certificate for a given hostname
-X509 *CertificateAuthority::generateCertificate(const char *commonname, struct ca_serial *cser)
+X509 *CertificateAuthority::generateCertificate(const char *commonname, struct ca_serial *cser, bool is_ip)
 {
     //create a blank cert
     ERR_clear_error();
     X509 *newCert = X509_new();
     if (newCert == NULL) {
-#ifdef DGDEBUG
-        std::cout  << thread_id << "new blank cert failed for " << commonname << std::endl;
-#endif
         log_ssl_errors("new blank cert failed for %s", commonname);
         return NULL;
     }
 
     ERR_clear_error();
     if (X509_set_version(newCert, 2) < 1) {
-#ifdef DGDEBUG
-        std::cout  << thread_id << "set_version on cert failed for " << commonname << std::endl;
-#endif
         log_ssl_errors("set_version on cert failed for %s", commonname);
         X509_free(newCert);
         return NULL;
@@ -310,9 +293,6 @@ X509 *CertificateAuthority::generateCertificate(const char *commonname, struct c
     //set a serial on the cert
     ERR_clear_error();
     if (X509_set_serialNumber(newCert, (cser->asn)) < 1) {
-#ifdef DGDEBUG
-        std::cout  << thread_id << "set_serialNumber on cert failed for " << commonname << std::endl;
-#endif
         log_ssl_errors("set_serialNumber on cert failed for %s", commonname);
         X509_free(newCert);
         return NULL;
@@ -321,17 +301,13 @@ X509 *CertificateAuthority::generateCertificate(const char *commonname, struct c
     //set valid from and expires dates
     // now from fixed date - should ensure regenerated certs are same and that servers in loadbalanced arrary give same cert
     if (!ASN1_TIME_set(X509_get_notBefore(newCert), _ca_start)) {
-#ifdef DGDEBUG
-        std::cout  << thread_id << "get_notBefore on cert failed for " << commonname << std::endl;
-#endif
+        DEBUG_debug("get_notBefore on cert failed for ", commonname );
         X509_free(newCert);
         return NULL;
     }
 
     if (!ASN1_TIME_set(X509_get_notAfter(newCert), _ca_end)) {
-#ifdef DGDEBUG
-        std::cout  << thread_id << "get_notAfter on cert failed for " << commonname << std::endl;
-#endif
+        DEBUG_debug("get_notAfter on cert failed for ", commonname);
         X509_free(newCert);
         return NULL;
     }
@@ -340,9 +316,6 @@ X509 *CertificateAuthority::generateCertificate(const char *commonname, struct c
     //the private key data type also contains the pub key which is used below.
     ERR_clear_error();
     if (X509_set_pubkey(newCert, _certPrivKey) < 1) {
-#ifdef DGDEBUG
-        std::cout  << thread_id << "set_pubkey on cert failed for " << commonname << std::endl;
-#endif
         log_ssl_errors("set_pubkey on cert failed for %s", commonname);
         X509_free(newCert);
         return NULL;
@@ -352,9 +325,6 @@ X509 *CertificateAuthority::generateCertificate(const char *commonname, struct c
     ERR_clear_error();
     X509_NAME *name = X509_get_subject_name(newCert);
     if (name == NULL) {
-#ifdef DGDEBUG
-        std::cout  << thread_id << "get_subject_name on cert failed for " << commonname << std::endl;
-#endif
         log_ssl_errors("get_subject_name on cert failed for %s", commonname);
         X509_free(newCert);
         return NULL;
@@ -366,9 +336,6 @@ X509 *CertificateAuthority::generateCertificate(const char *commonname, struct c
         MBSTRING_ASC, (unsigned char *)commonname, -1, -1, 0);
 
     if (rc < 1) {
-#ifdef DGDEBUG
-        std::cout  << thread_id << "NAME_add_entry_by_txt on cert failed for " << commonname << std::endl;
-#endif
         log_ssl_errors("NAME_add_entry_by_txt on cert failed for %s", commonname);
     //    X509_NAME_free(name);
         X509_free(newCert);
@@ -379,9 +346,6 @@ X509 *CertificateAuthority::generateCertificate(const char *commonname, struct c
     ERR_clear_error();
     X509_NAME *subjectName = X509_get_subject_name(_caCert);
     if (subjectName == NULL) {
-#ifdef DGDEBUG
-        std::cout  << thread_id << "get_subject_name on ca_cert failed for " << commonname << std::endl;
-#endif
         log_ssl_errors("get_subject_name on ca_cert failed for %s", commonname);
         X509_free(newCert);
         return NULL;
@@ -389,18 +353,45 @@ X509 *CertificateAuthority::generateCertificate(const char *commonname, struct c
 
     ERR_clear_error();
     if (X509_set_issuer_name(newCert, subjectName) < 1) {
-#ifdef DGDEBUG
-        std::cout  << thread_id << "set_issuer_name on cert failed for " << commonname << std::endl;
-#endif
         log_ssl_errors("set_issuer_name on cert failed for %s", commonname);
      //   X509_NAME_free(subjectName);
         X509_free(newCert);
         return NULL;
     }
-    {
-    String temp1 = "DNS:";
+
+    String temp1;
+
+   // E2LOGGER_error("common name is ",commonname);
     String temp2 = commonname;
-    temp1 = temp1 + temp2;
+    if(temp2.contains(":")) {
+        bool not_first_one = false;
+        temp2 += ":";
+  //      E2LOGGER_error("temp2 is ",temp2);
+        while (!temp2.empty()) {
+            String temp3 = temp2.before(":");
+            temp2 = temp2.after(":");
+            if (not_first_one) {
+                temp1 += ", ";
+            }
+            not_first_one = true;
+            if (temp3.isIp())
+                temp1 += "IP:";
+            else
+                temp1 += "DNS:";
+            temp1 += temp3;
+        }
+ //       E2LOGGER_error("alt_name string is ",temp1);
+        char    *value = (char*) temp1.toCharArray();
+        if( !addExtension(newCert, NID_subject_alt_name, value))
+            log_ssl_errors("Error adding subjectAltName to the request", commonname);
+
+    } else {
+        if (is_ip)
+            temp1 = "IP:";
+        else
+            temp1 = "DNS:";
+    temp1 += temp2;
+//        E2LOGGER_error("alt_name string is ",temp1);
     char    *value = (char*) temp1.toCharArray();
      if( !addExtension(newCert, NID_subject_alt_name, value))
         log_ssl_errors("Error adding subjectAltName to the request", commonname);
@@ -410,24 +401,19 @@ X509 *CertificateAuthority::generateCertificate(const char *commonname, struct c
     //sign it using the ca
     ERR_clear_error();
     if (!X509_sign(newCert, _caPrivKey, EVP_sha256())) {
-#ifdef DGDEBUG
-        std::cout  << thread_id << "X509_sign on cert failed for " << commonname << std::endl;
-#endif
         log_ssl_errors("X509_sign on cert failed for %s", commonname);
         X509_free(newCert);
         return NULL;
     }
 
-#ifdef DGDEBUG
-    std::cout  << thread_id << "certificate create " << name << std::endl;
-#endif
+    DEBUG_debug("certificate create ", name );
 
     return newCert;
 }
 
 //sets cert to the certificate for commonname
 //returns true if the cert was loaded from cache / false if it was generated
-bool CertificateAuthority::getServerCertificate(const char *commonname, X509 **cert, struct ca_serial *caser)
+bool CertificateAuthority::getServerCertificate(const char *commonname, X509 **cert, struct ca_serial *caser, bool is_ip)
 {
 
     getSerial(commonname, caser);
@@ -441,17 +427,13 @@ bool CertificateAuthority::getServerCertificate(const char *commonname, X509 **c
     caser->filepath = strdup(filepath.c_str());
     caser->filename = strdup(path.c_str());
 
-#ifdef DGDEBUG
-    std::cout  << thread_id << "looking for cert " << path << std::endl;
-#endif
+    DEBUG_debug("looking for cert ", path);
     //check to see if there is a symlink to the file
     //	std::string path(_certLinks + filename);
     FILE *link = fopen(path.c_str(), "r");
 
     if (link != NULL) {
-#ifdef DGDEBUG
-        std::cout  << thread_id << "Certificate found" << std::endl;
-#endif
+        DEBUG_debug("Certificate found");
 
         //if there was then the certificate has already been created
         *cert = PEM_read_X509(link, NULL, NULL, NULL);
@@ -461,25 +443,17 @@ bool CertificateAuthority::getServerCertificate(const char *commonname, X509 **c
         //dont need to check the return as this returns null if it couldnt load a cert
         return true;
     } else {
-#ifdef DGDEBUG
-        std::cout  << thread_id << "Certificate not found. Creating one" << std::endl;
-#endif
+        DEBUG_debug("Certificate not found. Creating one");
 
         //generate a certificate
-        *cert = generateCertificate(commonname, caser);
+        *cert = generateCertificate(commonname, caser, is_ip);
         return false;
     }
 }
 
 EVP_PKEY *CertificateAuthority::getServerPkey()
 {
-#if OPENSSL_VERSION_NUMBER < 0x10100000L
-    //openssl is missing a EVP_PKEY_dup function so just up the ref count
-    //see http://www.mail-archive.com/openssl-users@openssl.org/msg17614.html
-    CRYPTO_add(&_certPrivKey->references, 1, CRYPTO_LOCK_EVP_PKEY);
-#else
     EVP_PKEY_up_ref(_certPrivKey);
-#endif
     return _certPrivKey;
 }
 
@@ -558,4 +532,7 @@ bool CertificateAuthority::addExtension(X509 *cert, int nid, char *value)
     return (result > 0) ? true : false;
 }
 
-#endif //__SSLMITM
+//String CertificateAuthority::ASN1TIME2String(ASN1_TIME *atime)
+//{
+
+//};
