@@ -1,6 +1,7 @@
 #ifndef __HPP_CERTIFICATEAUTHORITY
 #define __HPP_CERTIFICATEAUTHORITY
-#ifdef __SSLMITM
+
+#include "String.hpp"
 
 struct ca_serial {
     ASN1_INTEGER *asn;
@@ -36,14 +37,15 @@ class CertificateAuthority
         time_t caEnd);
 
     ~CertificateAuthority();
-    X509 *generateCertificate(const char *commonname, struct ca_serial *cser);
+    X509 *generateCertificate(const char *commonname, struct ca_serial *cser, bool is_ip = false);
     bool getSerial(const char *commonname, struct ca_serial *cser);
-    bool getServerCertificate(const char *commonname, X509 **cert, struct ca_serial *cser);
+    bool getServerCertificate(const char *commonname, X509 **cert, struct ca_serial *cser, bool is_ip = false);
     bool writeCertificate(const char *hostname, X509 *newCert, struct ca_serial *cser);
     EVP_PKEY *getServerPkey();
     bool free_ca_serial(struct ca_serial *cs);
+    String cert_start_stop_hash;
+    //String ASN1TIME2String(ASN1_TIME *atime);
 };
 
-#endif //__SSLMITM
 
 #endif //__HPP_CERTIFICATEAUTHORITY
