@@ -5,14 +5,14 @@
 // INCLUDES
 
 #ifdef HAVE_CONFIG_H
-#include "dgconfig.h"
+#include "e2config.h"
 #endif
 #include "DataBuffer.hpp"
 #include "HTTPHeader.hpp"
 #include "OptionContainer.hpp"
+#include "Logger.hpp"
 
 #include <sys/stat.h>
-#include <syslog.h>
 #include <algorithm>
 #include <cstdlib>
 #include <unistd.h>
@@ -25,25 +25,24 @@
 
 // DEFINES
 
-#define __DGHEADER_SENDALL 0
-#define __DGHEADER_SENDFIRSTLINE 1
-#define __DGHEADER_SENDREST 2
+#define __E2HEADER_SENDALL 0
+#define __E2HEADER_SENDFIRSTLINE 1
+#define __E2HEADER_SENDREST 2
 
 // GLOBALS
 
 extern OptionContainer o;
-extern thread_local std::string thread_id;
 
 // IMPLEMENTATION
 
 DataBuffer::DataBuffer()
-    : data(new char[1]), buffer_length(0)
+    : data(new char[1]), data_length(0)
 {
     data[0] = '\0';
 }
 
-DataBuffer::DataBuffer(const void *indata, off_t length)
-    : data(new char[length]), buffer_length(length)
+DataBuffer::DataBuffer(const void *indata, off_t length)     //not used! PIP
+    : data(new char[length]), data_length(length)
 {
     memcpy(data, indata, length);
 }
@@ -56,10 +55,11 @@ void DataBuffer::reset()
     data = new char[1];
     data[0] = '\0';
 
-    compresseddata = NULL;
 
+    compresseddata = nullptr;
+    data_length = 0;
     buffer_length = 0;
-    compressed_buffer_length = 0;
+    compressed_buffer_data_length = 0;
 
     if (tempfilefd > -1) {
         close(tempfilefd);
@@ -82,9 +82,9 @@ void DataBuffer::reset()
 DataBuffer::~DataBuffer()
 {
     delete[] data;
-    if (compresseddata != NULL) {
+    if (compresseddata != nullptr) {
         delete[] compresseddata;
-        compresseddata = NULL;
+        compresseddata = nullptr;
     }
     if (tempfilefd > -1) {
         close(tempfilefd);
@@ -96,20 +96,145 @@ DataBuffer::~DataBuffer()
     }
 }
 
-// swap back to a compressed version of the data, if one exits
+void DataBuffer::set_current_config (FOptionContainer *newfgc) {
+    fgc = newfgc;
+}
+
+//}
+// swap back to a compressed version of the data, if one exists
 // also delete uncompressed version
 // if body was decompressed but not modified, this can save bandwidth
 void DataBuffer::swapbacktocompressed()
 {
-    if (compresseddata != NULL && compressed_buffer_length > 0) {
+    if (compresseddata != nullptr && compressed_buffer_data_length> 0) {
         delete[] data;
-        buffer_length = compressed_buffer_length;
+        data_length = compressed_buffer_data_length;
         data = compresseddata;
-        compresseddata = NULL;
-        compressed_buffer_length = 0;
+
+        compresseddata = nullptr;
+        compressed_buffer_data_length = 0;
+        DEBUG_network("Compressed data size ", data_length);
     }
 }
 
+int DataBuffer::readInFromSocket(Socket *sock, int size, bool wantall, int &result) {
+
+    int rc;
+
+    DEBUG_trace("size:", size, " wantall:", wantall);
+    if (size < 1) {
+        E2LOGGER_error("read request is negative");
+        return -1;
+    }
+
+    if (wantall) {
+            if (!swappedtodisk) {
+                // if not swapped to disk and file is too large for RAM, then swap to disk
+                if (data_length > o.content.max_content_ramcache_scan_size) {
+                    DEBUG_debug("swapping to disk");
+                    tempfilefd = getTempFileFD();
+                    if (tempfilefd < 0) {
+                        E2LOGGER_error("error buffering to disk so skipping disk buffering");
+                        result = DB_TOBIG;
+                        return -1;
+                    }
+                    write(tempfilefd, data, data_length);
+                    swappedtodisk = true;
+                    tempfilesize = data_length;
+                }
+            } else if (tempfilesize > o.content.max_content_filecache_scan_size) {
+                // if swapped to disk and file too large for that too, then give up
+                DEBUG_debug("defaultdm: file too big to be scanned, halting download");
+                result = DB_TOBIG | DB_TOBIG_SCAN;
+                return -1;
+            }
+        } else {
+            if (data_length > o.content.max_content_filter_size) {
+                // if we aren't downloading for virus scanning, and file too large for filtering, give up
+                DEBUG_debug("defaultdm: file too big to be filtered, halting download");
+                result = DB_TOBIG | DB_TOBIG_FILTER;
+                return -1;
+            }
+        }
+
+    DEBUG_debug("swappedtodisk:", swappedtodisk);
+        if (!swappedtodisk) {
+        if (size > (buffer_length - data_length)) {
+            if(!increase_buffer(size - (buffer_length - data_length))) {
+                size = (buffer_length - data_length);
+                if (size == 0) {
+                    result = DB_TOBIG;
+                    return -1;    // it is too big
+                }
+            }
+        }
+            if (chunked) {
+            DEBUG_debug("readChunk:", data_length, ",", size);
+                rc = sock->readChunk((data + data_length), size, timeout);
+            } else {
+            DEBUG_debug("bufferReadFromSocket:", data_length, ",", size);
+                rc = bufferReadFromSocket(sock, (data + data_length), size, timeout);
+            }
+
+            if (rc <= 0) {
+                if (chunked)
+                    got_all = true;
+                return -1;
+                // an error occurred so end the while()
+                // or none received so pipe iis closed or chunking has ended
+            } else {
+                bytes_toget -= rc;
+                data_length += rc;
+                data[data_length] = '\0';
+            }
+        } else {
+            if (chunked) {
+                rc = sock->readChunk(data, buffer_length, timeout);
+            } else {
+                rc = bufferReadFromSocket(sock, data,
+                        // if not getting everything until connection close, grab only what is left
+                                             (!geteverything && (bytes_toget  < buffer_length) ? bytes_toget
+                                                                                                    : buffer_length),
+                                             timeout);
+            }
+            if (rc <= 0) {
+                if (chunked)
+                    got_all = true;
+                result = 0;
+                return 0;
+            } else {
+                bytes_toget  -= rc;
+                write(tempfilefd, data, rc);
+                tempfilesize += rc;
+                DEBUG_debug("written to disk:", rc, " total:", tempfilesize);
+            }
+        }
+        result = 0;
+    DEBUG_debug("rc=", rc, " bytes_toget=", bytes_toget, " data_length=", data_length);
+        return rc;
+}
+
+bool DataBuffer::increase_buffer(int extra) {
+    int more = 65536;
+    if (extra > more)
+        more = extra;
+    if ((buffer_length + more) > o.content.max_content_filter_size) {
+        more = o.content.max_content_filter_size - buffer_length;
+    }
+    if (more > 0) {
+        char *temp = new char[buffer_length + more + 1]; // replacement store
+        temp[buffer_length + more] = '\0';
+        memcpy(temp, data, data_length); // copy the current data
+        delete[] data; // delete the current data block
+        data = temp;
+        temp = nullptr;
+        buffer_length += more; // update data size counter
+        DEBUG_debug("data buffer extended by ", more, " to ", buffer_length);
+        return true;
+    }
+    return false;
+}
+
 // a much more efficient reader that does not assume the contents of
 // the buffer gets filled thus reducing memcpy()ing and new()ing
 int DataBuffer::bufferReadFromSocket(Socket *sock, char *buffer, int size, int sockettimeout)
@@ -117,7 +242,7 @@ int DataBuffer::bufferReadFromSocket(Socket *sock, char *buffer, int size, int s
         int pos = 0;
         int rc;
         while (pos < size) {
-            rc = sock->readFromSocket(&buffer[pos], size - pos, 0, sockettimeout);
+            rc = sock->readFromSocket(&buffer[pos], size - pos, 0, sockettimeout, true);
             if (rc < 1) {
                 // none recieved or an error
                 if (pos > 0) {
@@ -136,7 +261,7 @@ int DataBuffer::bufferReadFromSocket(Socket *sock, char *buffer, int size, int s
 // incorporates a "global" timeout within which all reads must complete.
 int DataBuffer::bufferReadFromSocket(Socket *sock, char *buffer, int size, int sockettimeout, int stimeout)
 {
-
+    DEBUG_trace("");
     int pos = 0;
     int rc;
     struct timeval starttime;
@@ -146,7 +271,7 @@ int DataBuffer::bufferReadFromSocket(Socket *sock, char *buffer, int size, int s
         if (chunked) {
             rc = sock->readChunk(&buffer[pos], size - pos,sockettimeout );
         } else {
-            rc = sock->readFromSocket(&buffer[pos], size - pos, 0, sockettimeout, false);
+            rc = sock->readFromSocket(&buffer[pos], size - pos, 0, sockettimeout, true);
         }
         if (rc < 1) {
             // none recieved or an error
@@ -158,9 +283,7 @@ int DataBuffer::bufferReadFromSocket(Socket *sock, char *buffer, int size, int s
         pos += rc;
         gettimeofday(&nowadays, NULL);
         if (nowadays.tv_sec - starttime.tv_sec > stimeout) {
-#ifdef DGDEBUG
-            std::cerr << thread_id << "buffered socket read more than timeout" << std::endl;
-#endif
+            DEBUG_debug("buffered socket read more than timeout");
             return pos; // just return how much got so far then
         }
     }
@@ -173,17 +296,13 @@ int DataBuffer::getTempFileFD()
     if (tempfilefd > -1) {
         return tempfilefd;
     }
-    tempfilepath = o.download_dir.c_str();
+    tempfilepath = o.content.download_dir.c_str();
     tempfilepath += "/tfXXXXXX";
     char *tempfilepatharray = new char[tempfilepath.length() + 1];
     strcpy(tempfilepatharray, tempfilepath.toCharArray());
-    //	umask(S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP); // this mask is reversed
     umask(0007); // only allow access to e2g user and group
     if ((tempfilefd = mkstemp(tempfilepatharray)) < 0) {
-#ifdef DGDEBUG
-        std::cerr << thread_id << "error creating temp " << tempfilepath << ": " << strerror(errno) << std::endl;
-#endif
-        syslog(LOG_ERR, "%sCould not create temp file to store download for scanning: %s", thread_id.c_str(), strerror(errno));
+        E2LOGGER_error("Could not create temp file to store download for scanning: ", strerror(errno));
         tempfilefd = -1;
         tempfilepath = "";
     } else {
@@ -194,7 +313,7 @@ int DataBuffer::getTempFileFD()
 }
 
 // check the client's user agent, see if we have a DM plugin compatible with it, and use it to download the body of the given request
-bool DataBuffer::in(Socket *sock, Socket *peersock, HTTPHeader *requestheader, HTTPHeader *docheader, bool runav, int *headersent)
+bool DataBuffer::in(Socket *sock, Socket *peersock, HTTPHeader *requestheader, HTTPHeader *docheader, bool runav, int *headersent,StoryBoard &story, NaughtyFilter *cm)
 {
     //Socket *sock = where to read from
     //Socket *peersock = browser to send stuff to for keeping it alive
@@ -205,87 +324,68 @@ bool DataBuffer::in(Socket *sock, Socket *peersock, HTTPHeader *requestheader, H
     //				  or to mark that the header has already been sent
 
     // so we know if we only partially downloaded from
-    // squid so later, if allowed, we can send the rest
+    // upstream so later, if allowed, we can send the rest
     bool toobig = false;
 
     // match request to download manager so browsers potentially can have a prettier version
     // and software updates, stream clients, etc. can have a compatible version.
     //int rc = 0;
-#ifdef DGDEBUG
     int j = 0;
-#endif
  //   int rc = -1;
-    for (std::deque<Plugin *>::iterator i = o.dmplugins_begin; i != o.dmplugins_end; i++) {
-        if ((i + 1) == o.dmplugins_end) {
-#ifdef DGDEBUG
-            std::cerr << thread_id << "Got to final download manager so defaulting to always match." << std::endl;
-#endif
+    DEBUG_trace("");
+    for (std::deque<Plugin *>::iterator i = o.plugins.dmplugins_begin; i != o.plugins.dmplugins_end; i++) {
+        ++j;
+        if ((i + 1) == o.plugins.dmplugins_end) {
+            DEBUG_debug("Got to final download manager so defaulting to always match.");
             dm_plugin = (DMPlugin *)(*i);
             dm_plugin->in(this, sock, peersock, requestheader, docheader, runav, headersent, &toobig);
             break;
         } else {
-            if (((DMPlugin *)(*i))->willHandle(requestheader, docheader)) {
-#ifdef DGDEBUG
-                std::cerr << thread_id << "Matching download manager number: " << j << std::endl;
-#endif
-                dm_plugin = (DMPlugin *)(*i);
+            dm_plugin = (DMPlugin *)(*i);
+            if (story.runFunctEntry(dm_plugin->story_entry, *cm)) {
+                DEBUG_debug("Matching download manager number: ", j);
                 dm_plugin->in(this, sock, peersock, requestheader, docheader, runav, headersent, &toobig);
                 break;
             }
         }
-#ifdef DGDEBUG
         j++;
-#endif
     }
-    // we should check rc and log on error/warn
-    // note for later - Tue 16th November 2004
     return toobig;
 }
 
 // send the request body to the client after having been handled by a DM plugin
-bool DataBuffer::out(Socket *sock) //throw(std::exception)
+bool DataBuffer::out(Socket *sock)
 {
     if (dontsendbody) {
-#ifdef DGDEBUG
-        std::cerr << thread_id << "dontsendbody true; not sending" << std::endl;
-#endif
+        DEBUG_debug("dontsendbody true; not sending");
         return true;
     }
-    if (!(*sock).breadyForOutput(timeout)) return false; // exceptions on timeout or error
+  //  if (!(*sock).readyForOutput(timeout)) return false; // exceptions on timeout or error
 
+    DEBUG_trace("");
     if (tempfilefd > -1) {
-// must have been too big for ram so stream from disk in blocks
-#ifdef DGDEBUG
-        std::cerr << thread_id << "Sending " << tempfilesize - bytesalreadysent << " bytes from temp file (" << bytesalreadysent << " already sent)" << std::endl;
-#endif
+        // must have been too big for ram so stream from disk in blocks
+        DEBUG_debug("Sending ", tempfilesize - bytesalreadysent, " bytes from temp file (", bytesalreadysent, " already sent)");;
         off_t sent = bytesalreadysent;
         int rc;
 
         if (lseek(tempfilefd, bytesalreadysent, SEEK_SET) < 0)
             return false;
-//            throw std::runtime_error(std::string("Can't write to socket: ") + strerror(errno));
          int block_len;
         if(chunked)
             block_len = 4048;
         else
-            block_len = buffer_length;
+            block_len = data_length;
 
         while (sent < tempfilesize) {
-            rc = readEINTR(tempfilefd, data, block_len);
-#ifdef DGDEBUG
-            std::cerr << thread_id << "reading temp file rc:" << rc << std::endl;
-#endif
+            rc = read(tempfilefd, data, block_len);
+            DEBUG_debug("reading temp file rc:", rc);
             if (rc < 0) {
-#ifdef DGDEBUG
-                std::cerr << thread_id << "error reading temp file so throwing exception" << std::endl;
-#endif
+                DEBUG_debug("error reading temp file so throwing exception");
                 return false;
-    //            throw std::exception();
             }
             if (rc == 0) {
-#ifdef DGDEBUG
-                std::cerr << thread_id << "got zero bytes reading temp file" << std::endl;
-#endif
+                DEBUG_debug("got zero bytes reading temp file");
                 break; // should never happen
             }
             // as it's cached to disk the buffer must be reasonably big
@@ -298,9 +398,7 @@ bool DataBuffer::out(Socket *sock) //throw(std::exception)
                 }
             }
             sent += rc;
-#ifdef DGDEBUG
-            std::cerr << thread_id << "total sent from temp:" << sent << std::endl;
-#endif
+            DEBUG_debug("total sent from temp:", sent);
         }
         if (chunked && got_all) {
             String n;
@@ -313,26 +411,26 @@ bool DataBuffer::out(Socket *sock) //throw(std::exception)
         unlink(tempfilepath.toCharArray());
     } else {
         off_t sent = bytesalreadysent;
-#ifdef DGDEBUG
-        std::cerr << thread_id << "Sending " << buffer_length - bytesalreadysent << " bytes from RAM (" << buffer_length << " in buffer; " << bytesalreadysent << " already sent)" << std::endl;
-#endif
+        DEBUG_debug("Sending ", data_length, " bytes from RAM (", buffer_length, " in buffer; ", bytesalreadysent, " already sent)" );
         // it's in RAM, so just send it, no streaming from disk
         int block_len;
         if(chunked)
             block_len = 4048;
         else
-            block_len = buffer_length;
+            block_len = data_length;
 
-        if (buffer_length != 0) {
-            while (sent < buffer_length) {
-                if( block_len > (buffer_length - sent))
-                    block_len = (buffer_length - sent);
+        if (data_length != 0) {
+            while (sent < data_length) {
+                if( block_len > (data_length - sent))
+                    block_len = (data_length - sent);
                 if (chunked) {
-                    if (!sock->writeChunk(data + sent, block_len, timeout))
+                    if (!sock->writeChunk(data + sent, block_len, timeout)) {
+                        DEBUG_network("writeChunk failed after ", sent, " bytes");
                         return false;
+                    }
 
                 } else {
-                    if (!sock->writeToSocket(data + sent, buffer_length - sent, 0, timeout))
+                    if (!sock->writeToSocket(data + sent, data_length - sent, 0, timeout))
                         return false;
                 }
                 sent += block_len;
@@ -351,9 +449,7 @@ bool DataBuffer::out(Socket *sock) //throw(std::exception)
                     return false;
             }
         }
-#ifdef DGDEBUG
-        std::cerr << thread_id << "Sent " << buffer_length - bytesalreadysent << " bytes from RAM (" << buffer_length  << std::endl;
-#endif
+        DEBUG_debug("Sent ", data_length - bytesalreadysent," bytes from RAM (", buffer_length);
     }
     return true;
 }
@@ -361,12 +457,10 @@ bool DataBuffer::out(Socket *sock) //throw(std::exception)
 // zlib decompression
 void DataBuffer::zlibinflate(bool header)
 {
-    if (buffer_length < 12) {
+    if (data_length < 12) {
         return; // it can't possibly be zlib'd
     }
-#ifdef DGDEBUG
-    std::cerr << thread_id << "compressed size:" << buffer_length << std::endl;
-#endif
+    DEBUG_debug("compressed size:", data_length);
 
 #if ZLIB_VERNUM < 0x1210
 #warning ************************************
@@ -380,7 +474,7 @@ void DataBuffer::zlibinflate(bool header)
     }
 #endif
 
-    int newsize = buffer_length * 5; // good estimate of deflated HTML
+    int newsize = data_length * 5; // good estimate of deflated HTML
 
     char *block = new char[newsize];
     block[0] = '\0';
@@ -395,7 +489,7 @@ void DataBuffer::zlibinflate(bool header)
     d_stream.zfree = (free_func)0;
     d_stream.opaque = (voidpf)0;
     d_stream.next_in = (Bytef *)data;
-    d_stream.avail_in = buffer_length;
+    d_stream.avail_in = data_length;
     d_stream.next_out = (Bytef *)block;
     d_stream.avail_out = newsize;
 
@@ -408,51 +502,37 @@ void DataBuffer::zlibinflate(bool header)
 
     if (err != Z_OK) { // was a problem so just return
         delete[] block; // don't forget to free claimed memory
-#ifdef DGDEBUG
-        std::cerr << thread_id << "bad init inflate: " << err << std::endl;
-#endif
+        DEBUG_debug("bad init inflate: ", err);
         return;
     }
     while (true) {
-#ifdef DGDEBUG
-        std::cerr << thread_id << "inflate loop" << std::endl;
-#endif
+        DEBUG_debug("inflate loop");
         err = inflate(&d_stream, Z_SYNC_FLUSH);
         bytesgot = d_stream.total_out;
         if (err == Z_STREAM_END) {
             err = inflateEnd(&d_stream);
             if (err != Z_OK) {
                 delete[] block;
-#ifdef DGDEBUG
-                std::cerr << thread_id << "bad inflateEnd: " << d_stream.msg << std::endl;
-#endif
+                DEBUG_debug("bad inflateEnd: ", d_stream.msg);
                 return;
             }
             break;
         }
         if (err != Z_OK) { // was a problem so just return
             delete[] block; // don't forget to free claimed memory
-#ifdef DGDEBUG
-            std::cerr << thread_id << "bad inflate: " << d_stream.msg << std::endl;
-#endif
+            DEBUG_debug("bad inflate: ", d_stream.msg);
             err = inflateEnd(&d_stream);
             if (err != Z_OK) {
-#ifdef DGDEBUG
-                std::cerr << thread_id << "bad inflateEnd: " << d_stream.msg << std::endl;
-#endif
+                DEBUG_debug("bad inflateEnd: ", d_stream.msg);
             }
             return;
         }
-        if (bytesgot > o.max_content_filter_size) {
+        if (bytesgot > o.content.max_content_filter_size) {
             delete[] block; // don't forget to free claimed memory
-#ifdef DGDEBUG
-            std::cerr << thread_id << "inflated file larger than maxcontentfiltersize, not inflating further" << std::endl;
-#endif
+            DEBUG_debug("inflated file larger than maxcontentfiltersize, not inflating further");
             err = inflateEnd(&d_stream);
             if (err != Z_OK) {
-#ifdef DGDEBUG
-                std::cerr << thread_id << "bad inflateEnd: " << d_stream.msg << std::endl;
-#endif
+                DEBUG_debug("bad inflateEnd: ", d_stream.msg);
             }
             return;
         }
@@ -470,11 +550,9 @@ void DataBuffer::zlibinflate(bool header)
     }
 
     compresseddata = data;
-    compressed_buffer_length = buffer_length;
+    compressed_buffer_data_length = data_length;  // change from buffer_length
     buffer_length = bytesgot;
-#ifdef DGDEBUG
-    std::cerr << thread_id << "decompressed size: " << buffer_length << std::endl;
-#endif
+    DEBUG_debug("decompressed size: ", data_length);
     data = new char[bytesgot + 1];
     data[bytesgot] = '\0';
     memcpy(data, block, bytesgot);
@@ -489,9 +567,7 @@ struct newreplacement {
 bool DataBuffer::contentRegExp(FOptionContainer* &foc)
 {
 
-#ifdef DGDEBUG
-    std::cerr << thread_id << "Starting content reg exp replace" << std::endl;
-#endif
+    DEBUG_debug("Starting content reg exp replace");
     bool contentmodified = false;
     unsigned int i;
     unsigned int j, k, m;
@@ -563,15 +639,13 @@ bool DataBuffer::contentRegExp(FOptionContainer* &foc)
             }
 
             // now we know eventual size of content-replaced block, allocate memory for it
-            newblock = new char[buffer_length + sizediff + 1];
-            newblock[buffer_length + sizediff] = '\0';
+            newblock = new char[data_length + sizediff + 1];
+            newblock[data_length + sizediff] = '\0';
             srcoff = 0;
             dstpos = newblock;
             matches = m;
 
-#ifdef DGDEBUG
-            std::cerr << thread_id << "content matches:" << matches << std::endl;
-#endif
+            DEBUG_debug("content matches:", matches);
             // replace top-level matches using filled-out replacement strings
             newreplacement *newrep;
             for (j = 0; j < matches; j++) {
@@ -589,12 +663,12 @@ bool DataBuffer::contentRegExp(FOptionContainer* &foc)
                 delete newrep;
                 matchqueue.pop();
             }
-            if (srcoff < buffer_length) {
-                memcpy(dstpos, data + srcoff, buffer_length - srcoff);
+            if (srcoff < data_length) {
+                memcpy(dstpos, data + srcoff, data_length - srcoff);
             }
             delete[] data;
             data = newblock;
-            buffer_length = buffer_length + sizediff;
+            data_length = data_length + sizediff;
             contentmodified = true;
         }
     }
