@@ -5,23 +5,21 @@
 // INCLUDES
 
 #ifdef HAVE_CONFIG_H
-#include "dgconfig.h"
+#include "e2config.h"
 #endif
 #include "String.hpp"
 #include "OptionContainer.hpp"
 #include "NaughtyFilter.hpp"
 #include "RegExp.hpp"
 #include "ListContainer.hpp"
+#include "Logger.hpp"
 
 #include <cstring>
-#include <syslog.h>
 #include <algorithm>
 
 // GLOBALS
 
 extern OptionContainer o;
-extern thread_local std::string thread_id;
-
 
 #ifdef HAVE_PCRE
 extern RegExp absurl_re, relurl_re;
@@ -66,6 +64,14 @@ NaughtyFilter::NaughtyFilter(HTTPHeader &request, HTTPHeader &response)
     reset();
 }
 
+NaughtyFilter::NaughtyFilter(HTTPHeader &request, HTTPHeader &response, auth_rec &authrecin)
+{
+    request_header = &request;
+    response_header = &response;
+    authrec = &authrecin;
+    ch_isiphost.comp(",*[a-z|A-Z].*");
+    reset();
+}
 void NaughtyFilter::setURL(bool set_ismitm) {
     // do all of this normalisation etc just the once at the start.
     url = request_header->getUrl(false, false);
@@ -113,7 +119,8 @@ void NaughtyFilter::setURL(String &sni) {
 
 void NaughtyFilter::reset()
 {
-    anon_log = o.anonymise_logs;
+    anon_user = o.log.anonymise_logs;
+    anon_url = false;
     logurl = "";
     isItNaughty = false;
     isException = false;
@@ -138,10 +145,13 @@ void NaughtyFilter::reset()
     issiteonly = false;
 
     gomitm = false;
+    nomitm = false;
+    automitm = false;
     // resets from CH
     waschecked = false; // flags
     wasrequested = false;
     isexception = false;
+    issemiexception = false;
     isourwebserver = false;
     wasclean = false;
     isbypass = false;
@@ -167,11 +177,14 @@ void NaughtyFilter::reset()
     urlredirect = false;
     logcategory = false;
     upfailure = false;
-    isdirect = o.no_proxy;  // always set if no proxy defined
+    isdirect = o.net.no_proxy;  // always set if no proxy defined
     //            authed = false;
     //            isbanneduser = false;
     mimetype = "-";
     docsize = 0; // to store the size of the returned document for logging
+    orig_ip = "";
+    orig_port = 0;
+    got_orig_ip = false;
 
 }
 
@@ -180,16 +193,13 @@ void NaughtyFilter::reset()
 void NaughtyFilter::checkme(const char *rawbody, off_t rawbodylen, const String *url,
     const String *domain, FOptionContainer* &foc, unsigned int phraselist, int limit, bool searchterms)
 {
-#ifdef DGDEBUG
-    if (searchterms)
-        std::cerr << thread_id << "Content flagged as search terms - disabling hex decoding, META/TITLE extraction & HTML removal" << std::endl;
-#endif
+    if (searchterms) {
+        DEBUG_content("Content flagged as search terms - disabling hex decoding, META/TITLE extraction & HTML removal");
+    }
 
 
     if (foc->weighted_phrase_mode == 0) {
-#ifdef DGDEBUG
-        std::cerr << thread_id << "Weighted phrase mode 0 - not going any further." << std::endl;
-#endif
+        DEBUG_content("Weighted phrase mode 0 - not going any further.");
         return;
     }
 
@@ -203,10 +213,8 @@ void NaughtyFilter::checkme(const char *rawbody, off_t rawbodylen, const String
     // Do this now, as it's not especially case-sensitive,
     // and the case alteration should modify case post-decoding
     // Search terms are already hex decoded, as they need to be to strip URL decoding
-    if (!searchterms && o.hex_decode_content) { // Mod suggested by AFN Tue 8th April 2003
-#ifdef DGDEBUG
-        std::cerr << thread_id << "Hex decoding is enabled" << std::endl;
-#endif
+    if (!searchterms && o.naughty.hex_decode_content) { // Mod suggested by AFN Tue 8th April 2003
+        DEBUG_content("Hex decoding is enabled");
         char *hexdecoded_buf = new char[rawbodylen + 128 + 1];
         memset(hexdecoded_buf, 0, rawbodylen + 128 + 1);
         unsigned char c1;
@@ -247,16 +255,14 @@ void NaughtyFilter::checkme(const char *rawbody, off_t rawbodylen, const String
 
     // scan twice, with & without case conversion (if desired) - aids support for exotic char encodings
     // TODO: move META/title sentinel location outside this loop, as they are not case sensitive operations
-    bool preserve_case = o.preserve_case;
+    bool preserve_case = (o.naughty.preserve_case != 0);
     bool do_raw = false;
     bool do_nohtml = false;
 
-    if (o.preserve_case == 2) {
-// scanning twice *is* desired
-// first time round the loop, don't preserve case (non-exotic encodings)
-#ifdef DGDEBUG
-        std::cerr << thread_id << "Filtering with/without case preservation is enabled" << std::endl;
-#endif
+    if (o.naughty.preserve_case == 2) {
+        // scanning twice *is* desired
+        // first time round the loop, don't preserve case (non-exotic encodings)
+        DEBUG_content("Filtering with/without case preservation is enabled");
         preserve_case = false;
     }
 
@@ -269,33 +275,30 @@ void NaughtyFilter::checkme(const char *rawbody, off_t rawbodylen, const String
     // Store for the tag-stripped data
     // Don't bother tag stripping search terms
     char *bodynohtml = NULL;
-    if (!searchterms && (o.phrase_filter_mode == 1 || o.phrase_filter_mode == 2)) {
+    if (!searchterms && (o.naughty.phrase_filter_mode == 1 || o.naughty.phrase_filter_mode == 2)) {
         do_nohtml = true;
         bodynohtml = new char[hexdecodedlen + 128 + 1];
         memset(bodynohtml, 0, hexdecodedlen + 128 + 1);
     }
 
-    if ((o.phrase_filter_mode == 0 || o.phrase_filter_mode == 2 || o.phrase_filter_mode == 3))
+    if ((o.naughty.phrase_filter_mode == 0 || o.naughty.phrase_filter_mode == 2 || o.naughty.phrase_filter_mode == 3))
         do_raw = true;
 
-    for (int loop = 0; loop < (o.preserve_case == 2 ? 2 : 1); loop++) {
-#ifdef DGDEBUG
-        std::cerr << thread_id << "Preserve case: " << preserve_case << std::endl;
-#endif
+    for (int loop = 0; loop < (o.naughty.preserve_case == 2 ? 2 : 1); loop++) {
+        DEBUG_content("Preserve case: ", preserve_case );
 
         off_t i, j;
-#ifdef DGDEBUG
         if (searchterms || do_raw)
-            std::cerr << thread_id << "Raw content needed" << std::endl;
-#endif
+            DEBUG_content("Raw content needed");
+
         // use the one that's been hex decoded, but not stripped
         // make a copy of the document lowercase char by char
         if (preserve_case) {
-            if (do_nohtml || o.phrase_filter_mode == 3) {
+            if (do_nohtml || o.naughty.phrase_filter_mode == 3) {
                 for (i = 0; i < hexdecodedlen; i++) {
                     c = hexdecoded[i];
                     //				if (c == 13 || c == 9 || c == 10) {
-                    if (c < 46 || c == 58 || c == 59 || c == 63 || (c > 90 && c < 97)) {
+                    if (c < '/' || c == ':' || c == ';' || c == '=' || c == '?' || c == '@' || (c > 90 && c < 97)) {
                         c = 32; // convert all whitespace and most punctuation marks to a space
                     }
                     bodylc[i] = c;
@@ -304,17 +307,15 @@ void NaughtyFilter::checkme(const char *rawbody, off_t rawbodylen, const String
                 for (i = 0; i < hexdecodedlen; i++) {
                     c = hexdecoded[i];
                     //				if (c == 13 || c == 9 || c == 10) {
-                    if (c < 46 || (c > 57 && c < 65) || (c > 90 && c < 97)) {
+                    if (c < '/' || (c > '9' && c < 'A') || (c > 'Z' && c < 'a')) {
                         c = 32; // convert all whitespace and most punctuation marks to a space
                     }
                     bodylc[i] = c;
                 }
             }
         } else {
-#ifdef DGDEBUG
-            std::cerr << thread_id << "Not preserving case of raw content" << std::endl;
-#endif
-            if (do_nohtml || o.phrase_filter_mode == 3) {
+            DEBUG_content("Not preserving case of raw content");
+            if (do_nohtml || o.naughty.phrase_filter_mode == 3) {
                 for (i = 0; i < hexdecodedlen; i++) {
                     c = hexdecoded[i];
                     if (c >= 'A' && c <= 'Z') {
@@ -323,7 +324,7 @@ void NaughtyFilter::checkme(const char *rawbody, off_t rawbodylen, const String
                         c += 32; // 224 + c - 192
                     } else {
                         //if (c == 13 || c == 9 || c == 10) {
-                        if (c < 46 || c == 58 || c == 59 || c == 63 || (c > 90 && c < 97)) {
+                        if (c < '/' || c == ':' || c == ';' || c == '=' || c == '?' || c == '@' || (c > 90 && c < 97)) {
                             c = 32; // convert all whitespace and most punctuation marks to a space
                         }
                     }
@@ -338,7 +339,7 @@ void NaughtyFilter::checkme(const char *rawbody, off_t rawbodylen, const String
                         c += 32; // 224 + c - 192
                     } else {
                         //if (c == 13 || c == 9 || c == 10) {
-                        if (c < 46 || (c > 57 && c < 65) || (c > 90 && c < 97)) {
+                        if (c < '/' || (c > '9' && c < 'A') || (c > 'Z' && c < 'a')) {
                             c = 32; // convert all whitespace and most punctuation marks to a space
                         }
                     }
@@ -349,41 +350,31 @@ void NaughtyFilter::checkme(const char *rawbody, off_t rawbodylen, const String
 
         // filter meta tags & title only
         // based on idea from Nicolas Peyrussie
-        if (!searchterms && (o.phrase_filter_mode == 3)) {
-#ifdef DGDEBUG
-            std::cerr << thread_id << "Filtering META/title" << std::endl;
-#endif
+        if (!searchterms && (o.naughty.phrase_filter_mode == 3)) {
+            DEBUG_content("Filtering META/title");
             bool addit = false; // flag if we should copy this char to filtered version
             bool needcheck = false; // flag if we actually find anything worth filtering
             off_t bodymetalen;
 
             // find </head> or <body> as end of search range
             char *endhead = strstr(bodylc, "</head");
-#ifdef DGDEBUG
             if (endhead != NULL)
-                std::cerr << thread_id << "Found '</head', limiting search range" << std::endl;
-#endif
+                DEBUG_content("Found '</head', limiting search range");
             if (endhead == NULL) {
                 endhead = strstr(bodylc, "<body");
-#ifdef DGDEBUG
                 if (endhead != NULL)
-                    std::cerr << thread_id << "Found '<body', limiting search range" << std::endl;
-#endif
+                    DEBUG_content("Found '<body', limiting search range");
             }
 
             // if case preserved, also look for uppercase versions
             if (preserve_case and (endhead == NULL)) {
                 endhead = strstr(bodylc, "</HEAD");
-#ifdef DGDEBUG
                 if (endhead != NULL)
-                    std::cerr << thread_id << "Found '</HEAD', limiting search range" << std::endl;
-#endif
+                    DEBUG_content("Found '</HEAD', limiting search range");
                 if (endhead == NULL) {
                     endhead = strstr(bodylc, "<BODY");
-#ifdef DGDEBUG
                     if (endhead != NULL)
-                        std::cerr << thread_id << "Found '<BODY', limiting search range" << std::endl;
-#endif
+                        DEBUG_content("Found '<BODY', limiting search range");
                 }
             }
 
@@ -402,9 +393,7 @@ void NaughtyFilter::checkme(const char *rawbody, off_t rawbodylen, const String
                 // are we at the start of a tag?
                 if ((!addit) && (c == '<')) {
                     if ((strncmp(bodylc + i + 1, "meta", 4) == 0) or (preserve_case and (strncmp(bodylc + i + 1, "META", 4) == 0))) {
-#ifdef DGDEBUG
-                        std::cerr << thread_id << "Found META" << std::endl;
-#endif
+                        DEBUG_content("Found META");
                         // start adding data to the check buffer
                         addit = true;
                         needcheck = true;
@@ -414,9 +403,7 @@ void NaughtyFilter::checkme(const char *rawbody, off_t rawbodylen, const String
                     }
                     // are we at the start of a title tag?
                     else if ((strncmp(bodylc + i + 1, "title", 5) == 0) or (preserve_case and (strncmp(bodylc + i + 1, "TITLE", 5) == 0))) {
-#ifdef DGDEBUG
-                        std::cerr << thread_id << "Found TITLE" << std::endl;
-#endif
+                        DEBUG_content("Found TITLE");
                         // start adding data to the check buffer
                         addit = true;
                         needcheck = true;
@@ -437,11 +424,11 @@ void NaughtyFilter::checkme(const char *rawbody, off_t rawbodylen, const String
                 if (addit) {
                     // if we're in "record" mode (i.e. inside a title/metatag), strip certain characters out
                     // of the data (to sanitise metatags & aid filtering of titles)
-                    if (c == ',' || c == '=' || c == '"' || c == '\''
-                        || c == '(' || c == ')' || c == '.') {
-                        // replace with a space
-                        c = 32;
-                    }
+                    //if (c == ',' || c == '=' || c == '"' || c == '\''
+                    //    || c == '(' || c == ')' || c == '.') {
+                    //    // replace with a space
+                    //    c = 32;
+                    //}
                     // don't bother duplicating spaces
                     if ((c != 32) || (c == 32 && (bodymeta[j - 1] != 32))) {
                         bodymeta[j++] = c; // copy it to the filtered copy
@@ -450,16 +437,12 @@ void NaughtyFilter::checkme(const char *rawbody, off_t rawbodylen, const String
             }
             if (needcheck) {
                 bodymeta[j++] = '\0';
-#ifdef DGDEBUG
-                std::cerr << bodymeta << std::endl;
-#endif
+                DEBUG_content(bodymeta);
                 bodymetalen = j;
                 checkphrase(bodymeta, bodymetalen, NULL, NULL, foc, phraselist, limit, searchterms);
             }
-#ifdef DGDEBUG
             else
-                std::cerr << thread_id << "Nothing to filter" << std::endl;
-#endif
+                DEBUG_content("Nothing to filter");
 
             delete[] bodymeta;
             // surely the intention is to search *only* meta/title, so always exit
@@ -471,11 +454,9 @@ void NaughtyFilter::checkme(const char *rawbody, off_t rawbodylen, const String
         }
 
         if (do_nohtml) {
-// if we fell through to here, use the one that's been hex decoded AND stripped
-// Strip HTML
-#ifdef DGDEBUG
-            std::cerr << thread_id << "\"Smart\" filtering is enabled" << std::endl;
-#endif
+            // if we fell through to here, use the one that's been hex decoded AND stripped
+            // Strip HTML
+            DEBUG_content("\"Smart\" filtering is enabled");
             // we need this extra byte *
             bool inhtml = false; // to flag if our pointer is within a html <>
             bool addit; // flag if we should copy this char to filtered version
@@ -503,9 +484,7 @@ void NaughtyFilter::checkme(const char *rawbody, off_t rawbodylen, const String
                     bodynohtml[j++] = c; // copy it to the filtered copy
                 }
             }
-#ifdef DGDEBUG
-            std::cerr << thread_id << "Checking smart content" << std::endl;
-#endif
+            DEBUG_content("Checking smart content");
             checkphrase(bodynohtml, j - 1, NULL, NULL, foc, phraselist, limit, searchterms);
             if (isItNaughty || isException) {
                 delete[] bodylc;
@@ -523,9 +502,7 @@ void NaughtyFilter::checkme(const char *rawbody, off_t rawbodylen, const String
                 delete[] hexdecoded;
             return; // only doing nohtml mode filtering
         } else {
-#ifdef DGDEBUG
-            std::cerr << thread_id << "Checking raw content" << std::endl;
-#endif
+            DEBUG_content("Checking raw content");
 
             if (do_nohtml) { // already removed if not!
                 // replace html tag start and finish with space so that Start and finish words are detected
@@ -606,17 +583,15 @@ void NaughtyFilter::checkphrase(char *file, off_t filelen, const String *url, co
 
         // check for absolute URLs
         if (absurl_re.match(file, Rre)) {
-// each match generates 2 results (because of the brackets in the regex), we're only interested in the first
-#ifdef DGDEBUG
-            std::cerr << thread_id << "Found " << Rre.numberOfMatches() / 2 << " absolute URLs:" << std::endl;
-#endif
+            // each match generates 2 results (because of the brackets in the regex), 
+            // we're only interested in the first
+            DEBUG_content("Found ", Rre.numberOfMatches() / 2, " absolute URLs:");
             for (int i = 0; i < Rre.numberOfMatches(); i += 2) {
                 // chop off quotes
                 u = Rre.result(i);
                 u = u.subString(1, u.length() - 2);
-#ifdef DGDEBUG
-                std::cerr << u << std::endl;
-#endif
+                DEBUG_content(u);
+
                 if ((((j = foc->inBannedSiteList(u, false, false, false, lastcategory)) != NULL) &&
                     !(lastcategory.contains("ADs")))
                     || (((j = foc->inBannedURLList(u, false, false,false, lastcategory )) != NULL) &&
@@ -660,10 +635,9 @@ void NaughtyFilter::checkphrase(char *file, off_t filelen, const String *url, co
             if (currurl[currurl.length() - 1] != '/')
                 currurl += "/";
 
-// each match generates 2 results (because of the brackets in the regex), we're only interested in the first
-#ifdef DGDEBUG
-            std::cerr << thread_id << "Found " << Rre.numberOfMatches() / 2 << " relative URLs:" << std::endl;
-#endif
+            // each match generates 2 results (because of the brackets in the regex),
+            // we're only interested in the first
+            DEBUG_content("Found ", Rre.numberOfMatches() / 2, " relative URLs:");
             for (int i = 0; i < Rre.numberOfMatches(); i += 2) {
                 u = Rre.result(i);
 
@@ -672,9 +646,7 @@ void NaughtyFilter::checkphrase(char *file, off_t filelen, const String *url, co
                 if (u.contains("://"))
                     continue;
 
-#ifdef DGDEBUG
-                std::cerr << u << std::endl;
-#endif
+                DEBUG_content(u);
                 // remove src/href & quotes
                 u = u.after("=");
                 u.removeWhiteSpace();
@@ -685,9 +657,9 @@ void NaughtyFilter::checkphrase(char *file, off_t filelen, const String *url, co
                     u = (*domain) + u;
                 else
                     u = currurl + u;
-#ifdef DGDEBUG
-                std::cerr << thread_id << "absolute form: " << u << std::endl;
-#endif
+
+                DEBUG_content("absolute form: ", u );
+
                 if ((((j = foc->inBannedSiteList(u, false, false, false, lastcategory)) != NULL) &&
                      !(lastcategory.contains("ADs")))
                     || (((j = foc->inBannedURLList(u, false, false, false, lastcategory)) != NULL) &&
@@ -721,10 +693,8 @@ void NaughtyFilter::checkphrase(char *file, off_t filelen, const String *url, co
         if (catinited) {
             weighting = ourcat->second.weight;
             weightedphrase += "]";
-#ifdef DGDEBUG
-            std::cerr << thread_id << weightedphrase << std::endl;
-            std::cerr << thread_id << "score from embedded URLs: " << ourcat->second.weight << std::endl;
-#endif
+            DEBUG_content(weightedphrase);
+            DEBUG_content("score from embedded URLs: ", ourcat->second.weight );
         }
     }
 #endif // END OF LEAVE_OUT_FOR_NOW
@@ -766,12 +736,12 @@ void NaughtyFilter::checkphrase(char *file, off_t filelen, const String *url, co
                 // check this time limit against the list of time limits
                 time = *(++combicurrent);
                 if (not(o.lm.l[phraselist]->checkTimeAtD(time))) {
-// nope - so don't take any notice of it
-#ifdef DGDEBUG
+                    // nope - so don't take any notice of it
+#ifdef DEBUG_LOW
                     combicurrent++;
                     cat = (*++combicurrent);
-                    std::cerr << thread_id << "Ignoring combi phrase based on time limits: " << combisofar << "; "
-                              << o.lm.l[phraselist]->getListCategoryAtD(cat) << std::endl;
+                    DEBUG_content("Ignoring combi phrase based on time limits: ", combisofar, "; ",
+                                    o.lm.l[phraselist]->getListCategoryAtD(cat));
 #else
                     combicurrent += 2;
 #endif
@@ -818,13 +788,10 @@ void NaughtyFilter::checkphrase(char *file, off_t filelen, const String *url, co
                     } else {
                         weightedphrase += combisofar;
                     }
-#ifdef DGDEBUG
-                    std::cerr << thread_id << "found combi weighted phrase (" << foc->weighted_phrase_mode << "): "
-                              << combisofar << " x" << lowest_occurrences << " (per phrase: "
-                              << weight << ", calculated: "
-                              << (weight * (foc->weighted_phrase_mode == 2 ? 1 : lowest_occurrences)) << ")"
-                              << std::endl;
-#endif
+
+                    DEBUG_content("found combi weighted phrase (", foc->weighted_phrase_mode, "): ", combisofar,
+                                 " x", lowest_occurrences, " (per phrase: ",  weight,
+                                 ", calculated: ", (weight * (foc->weighted_phrase_mode == 2 ? 1 : lowest_occurrences)), ")" );
 
                     weightedphrase += ")";
                     combisofar = "";
@@ -875,11 +842,8 @@ void NaughtyFilter::checkphrase(char *file, off_t filelen, const String *url, co
     while (foundcurrent != foundend) {
         // check time for current phrase
         if (not o.lm.l[phraselist]->checkTimeAt(foundcurrent->second.first)) {
-#ifdef DGDEBUG
-            std::cerr << thread_id << "Ignoring phrase based on time limits: "
-                      << foundcurrent->first << ", "
-                      << o.lm.l[phraselist]->getListCategoryAt(foundcurrent->second.first) << std::endl;
-#endif
+            DEBUG_content("Ignoring phrase based on time limits: ", foundcurrent->first,
+                             ", ", o.lm.l[phraselist]->getListCategoryAt(foundcurrent->second.first) );
             foundcurrent++;
             continue;
         }
@@ -910,7 +874,7 @@ void NaughtyFilter::checkphrase(char *file, off_t filelen, const String *url, co
                 }
             }
 
-            if (o.show_weighted_found) {
+            if (o.naughty.show_weighted_found) {
                 if (weightedphrase.length() > 0) {
                     weightedphrase += "+";
                 }
@@ -920,12 +884,10 @@ void NaughtyFilter::checkphrase(char *file, off_t filelen, const String *url, co
 
                 weightedphrase += foundcurrent->first;
             }
-#ifdef DGDEBUG
-            std::cerr << "found weighted phrase (" << foc->weighted_phrase_mode << "): "
-                      << foundcurrent->first << " x" << foundcurrent->second.second << " (per phrase: "
-                      << o.lm.l[phraselist]->getWeightAt(foundcurrent->second.first)
-                      << ", calculated: " << weight << ")" << std::endl;
-#endif
+            DEBUG_content("found weighted phrase (", foc->weighted_phrase_mode, "): ", foundcurrent->first,
+                            " x", foundcurrent->second.second,
+                            " (per phrase: ", o.lm.l[phraselist]->getWeightAt(foundcurrent->second.first),
+                            ", calculated: ", weight, ")" );
         } else if (type == -1) {
             isException = true;
             isItNaughty = false;
@@ -941,18 +903,14 @@ void NaughtyFilter::checkphrase(char *file, off_t filelen, const String *url, co
         foundcurrent++;
     }
 
-#ifdef DGDEBUG
-    std::cerr << thread_id << "WEIGHTING: " << weighting << std::endl;
-#endif
+    DEBUG_content("WEIGHTING: ", weighting );
 
     // store the lowest negative weighting or highest positive weighting out of all filtering runs, preferring to store positive weightings.
     if ((weighting < 0 && naughtiness <= 0 && weighting < naughtiness) || (naughtiness >= 0 && weighting > naughtiness) || (naughtiness < 0 && weighting > 0)) {
         naughtiness = weighting;
     }
 
-#ifdef DGDEBUG
-    std::cerr << thread_id << "NAUGHTINESS: " << naughtiness << std::endl;
-#endif
+    DEBUG_content("NAUGHTINESS: ", naughtiness );
 
     // *now* we can safely get down to the whole banning business!
 
@@ -984,23 +942,30 @@ void NaughtyFilter::checkphrase(char *file, off_t filelen, const String *url, co
         return;
     }
 
+    bool log_it = o.naughty.show_all_weighted_found;
+    auto mno = searchterms ? 454 : 402;
+
     if (weighting > limit) {
+        log_it = true;
         isItNaughty = true;
+        message_no = mno;
+        whatIsNaughty = o.language_list.getTranslation(searchterms ? 455 : 403);
+    }
+
+    if(log_it) {
         // Weighted phrase limit of
         // Weighted search term limit of
-        message_no = searchterms ? 454 : 402;
-        whatIsNaughtyLog = o.language_list.getTranslation(message_no);
+        whatIsNaughtyLog = o.language_list.getTranslation(mno);
         whatIsNaughtyLog += String(limit).toCharArray();
         whatIsNaughtyLog += " : ";
         whatIsNaughtyLog += String(weighting).toCharArray();
-        if (o.show_weighted_found) {
+        if (o.naughty.show_weighted_found) {
             whatIsNaughtyLog += " (";
             whatIsNaughtyLog += weightedphrase;
             whatIsNaughtyLog += ")";
         }
         // Weighted phrase limit exceeded.
         // Weighted search term limit exceeded.
-        whatIsNaughty = o.language_list.getTranslation(searchterms ? 455 : 403);
         // Generate category list, sorted with highest scoring first.
         bool nonempty = false;
         bool belowthreshold = false;
@@ -1037,8 +1002,8 @@ void NaughtyFilter::checkphrase(char *file, off_t filelen, const String *url, co
             }
         }
         whatIsNaughtyCategories = categories.toCharArray();
-        return;
     }
+        return;
     // whatIsNaughty is what is displayed in the browser
     // whatIsNaughtyLog is what is logged in the log file if at all
 }
@@ -1054,3 +1019,67 @@ bool NaughtyFilter::isIPHostnameStrip(String url)
         return true;
 //    return ldl->fg[0]->isIPHostname(url);
 }
+
+String NaughtyFilter::getFlags() {
+    String flags = listen_port;
+    flags += ":";
+    if (!(authrec == nullptr)) {
+        if (authrec->is_transparent)
+            flags += "T";
+        else if (authrec->is_icap)
+            flags += "I";
+        else {
+            if(authrec->is_tlsproxy)
+                flags += "L";
+            else
+                flags += "P";
+        }
+
+        if (o.log.addECHtoFlags && hasECH) {
+            flags += "E";
+        }
+
+        if (ismitm)
+            flags += "M";
+        else if (isconnect)
+            flags += "S";
+        else if (!authrec->is_icap)
+            flags += "H";
+
+        flags += ":";
+
+        flags += authrec->user_source;
+        flags += ":";
+        flags += authrec->group_source;
+    }
+    return flags;
+}
+
+String NaughtyFilter::get_lastmatch() {
+    if (anon_url) {
+        return lastmatch.anonimise();
+    }
+    return lastmatch;
+}
+
+String NaughtyFilter::get_logUrl() {
+    if (anon_url) {
+        String t1 = logurl.before("://");
+        String t2 = logurl.after("://");
+        t1 += "://";
+        t1 += t2.anonimise();
+        return t1;
+    }
+    return logurl;
+}
+
+String NaughtyFilter::main_category() {
+    String temp = whatIsNaughtyCategories;
+    if (temp.contains(",")) {
+        temp = temp.before(",");
+    }
+    if (temp == "-") temp = "";
+    temp.toLower();
+    return temp;
+}
+
