@@ -7,10 +7,11 @@
 // INCLUDES
 
 #ifdef HAVE_CONFIG_H
-#include "dgconfig.h"
+#include "e2config.h"
 #endif
 #include "String.hpp"
 #include "md5.hpp"
+#include "Logger.hpp"
 
 #if defined(__GNUC__) && __GNUC__ < 3 && __GNUC_MINOR__ < 96
 #warning "Using strstream instead of sstream"
@@ -22,7 +23,6 @@
 #include <cstdlib>
 #include <cstring>
 #include <memory>
-#include <syslog.h>
 
 extern thread_local std::string thread_id;
 
@@ -194,14 +194,14 @@ long int String::toLong()
 }
 
 // return integer from hex string
-int String::hexToInteger()
+long int String::hexToInteger()
 {
     int n = 0; // position in string
     int m = 0; // position in digit[] to shift
     int count; // loop index
-    int intValue = 0; // integer value of hex string
-    int digit[10]; // hold values to convert
-    while (n < 9) {
+    long int intValue = 0; // integer value of hex string
+    int digit[15]; // hold values to convert
+    while (n < 14) {
         if ((*this)[n] == '\0')
             break;
         if ((*this)[n] >= '0' && (*this)[n] <= '9')
@@ -268,6 +268,38 @@ void String::swapChar(char old, char newc)
     delete [] c;
 }
 
+void String::removeChar(char old)
+{
+    unsigned int l = this->length();
+    char *c = new char[l + 1];
+    unsigned int j = 0;
+    const char *d = this->c_str();
+    for (unsigned int i = 0; i < l; i++) {
+        if (d[i] != old) {
+            c[j++] = d[i];
+        }
+    }
+    c[j] = 0;
+    *this = String(c);
+    delete [] c;
+}
+
+void String::baseDir()
+{
+    size_t fnsize;
+    if ((fnsize = this->find_last_of("/")) > 0)
+        *this = this->subString(0,++fnsize);
+}
+
+void String::fullPath(String &base_dir) {
+    if(!this->startsWith("/"))
+    {
+        String temp(base_dir);
+        temp += *this;
+        *this = temp;
+    }
+}
+
 // decode %xx to individual characters (checkme: i'm sure this is duplicated elsewhere...)
 void String::hexDecode()
 {
@@ -399,12 +431,12 @@ bool String::headerVal()
 void String::chop()
 {
     if (this->length() > 0)
-    *this = this->substr(0, this->length() - 1);
+        *this = this->substr(0, this->length() - 1);
 }
 
 void String::lop()
 {
-        if (this->length() > 0)
+    if (this->length() > 0)
         *this = this->substr(1);
 }
 
@@ -420,6 +452,28 @@ void String::removeWhiteSpace()
     *this = this->substr(start, (end - start) + 1);
 }
 
+void String::removePunctuation() {
+    std::string temp;
+    bool lastspace = false;
+    unsigned char t;
+    unsigned int l = this->length();
+    for (unsigned int i = 0; i < l; i++) {
+        t = (*this)[i];
+        if (t < '/' || t == ':' || t == ';' || t == '=' || t == '?' || t == '@' || (t > 90 && t < 97)) {
+            t = 32; // convert all whitespace and most punctuation marks to a space
+        }
+        if (t == 32) {
+            if (lastspace) continue;
+            lastspace = true;
+        } else {
+            lastspace = false;
+        }
+            temp += t;
+            continue;
+    }
+    *this = temp;
+}
+
 // remove protocol specifier
 void String::removePTP()
 {
@@ -561,13 +615,14 @@ String String::md5()
 
 String String::sort_search()
 {
+    this->toLower();
     int ln = this->length();
     if (ln < 3) {
         return (*this);
     }
     //char *temp = new (std::nothrow) char[ln + 1];
     //if (temp == NULL)  {
-     //   syslog(LOG_ERR, "Unable to create temp char[%d] in sort_search", ln);
+     //  E2LOGGER_error("Unable to create temp char[%d] in sort_search", ln);
       //  return (*this);
     //}
    // strcpy(temp, (this)->c_str());
@@ -597,11 +652,12 @@ String String::sort_search()
         if ( j < ln) ind[j++] = 0;
         i++;
     };
-#ifdef DGDEBUG
+#ifdef DEBUG_LOW
     int k = 0;
     while (k <= c)
         {
-        std::cerr << thread_id << "Search word " << k << " is " << p[k] << std::endl;
+//        std::cerr << thread_id << "Search word " << k << " is " << p[k] << std::endl;
+        DEBUG_debug("Search word ", k, " is ", p[k]);
         k++;
         }
 #endif
@@ -622,11 +678,12 @@ String String::sort_search()
             j++;
         };
     };
-#ifdef DGDEBUG
+#ifdef DEBUG_LOW
     k = 0;
     while (k <= c)
         {
-        std::cerr << thread_id << "Search word after sort" << k << " is " << p[k] << std::endl;
+        //std::cerr << thread_id << "Search word after sort" << k << " is " << p[k] << std::endl;
+        DEBUG_debug("Search word after sort", k, " is ", p[k]);
         k++;
         }
 #endif
@@ -639,9 +696,7 @@ String String::sort_search()
         else
            ret = ret + "+" + p[j++];
     }
-#ifdef DGDEBUG
-    std::cerr << thread_id << "Search words after sort are "  << ret.c_str() << std::endl;
-#endif
+    DEBUG_debug("Search words after sort are ", ret);
    // delete [] temp;
     return ret;
 };
@@ -662,3 +717,48 @@ String String::CN() {
  //       return true;
   //  return false;
 //}
+
+String String::anonimise() {
+    std::string temp;
+    unsigned char t;
+    unsigned int l = this->length();
+    for (unsigned int i = 0; i < l; i++) {
+        t = (*this)[i];
+        if (!(t < '/' || t == ':' || t == ';' || t == '=' || t == '?' || t == '@' || (t > 90 && t < 97))) {
+            t = 'x'; // convert all alphanumeric to 'x'
+        }
+        temp += t;
+        continue;
+    }
+    String ts = temp;
+    return ts;
+}
+
+bool String::isIp() {
+    // Regex expression for validating IPv4
+    RegExp ipv4;
+    ipv4.comp("(([0-9]|[1-9][0-9]|1[0-9][0-9]|2[0-4][0-9]|25[0-5])\\.){3}([0-9]|[1-9][0-9]|1[0-9][0-9]|2[0-4][0-9]|25[0-5])");
+
+    RegResult rs;
+
+    if(ipv4.match((*this).c_str(), rs))
+        return true;
+
+    return false;
+
+    // Regex expression for validating IPv6
+   // regex ipv6("((([0-9a-fA-F]){1,4})\\:){7}([0-9a-fA-F]){1,4}");
+}
+
+bool String::is_valid_domain() {
+    unsigned int l = this->length();
+    const char *d = this->c_str();
+    if ((d[0] == '-')||(d[l - 1] == '-')) return false;  // hyphens are not allowed at start or end of domain
+    for (unsigned int i = 0; i < l; i++) {
+        if (!((d[i] >= '0' && d[i] <= '9') || (d[i] >= 'A' && d[i] <= 'Z') || (d[i] >= 'a' && d[i] <= 'z') ||
+              (d[i] == '-')||(d[i] == '.'))) {
+            return false;
+        }
+    }
+    return true;
+}
